@@ -31,7 +31,7 @@ from twisted.cred.checkers import InMemoryUsernamePasswordDatabaseDontUse
 from twisted.conch.interfaces import IConchUser
 from twisted.conch.error import ConchError, UserRejectedKey, HostKeyChanged
 
-if requireModule('Crypto.Cipher.AES') and requireModule('pyasn1.type'):
+if requireModule('cryptography') and requireModule('pyasn1.type'):
     from twisted.conch.ssh.factory import SSHFactory
     from twisted.conch.ssh.userauth import SSHUserAuthServer
     from twisted.conch.ssh.connection import SSHConnection
@@ -52,7 +52,7 @@ if requireModule('Crypto.Cipher.AES') and requireModule('pyasn1.type'):
 
     from twisted.conch.ssh.transport import SSHClientTransport
 else:
-    skip = "can't run w/o PyCrypto and pyasn1"
+    skip = "can't run w/o cryptography and pyasn1"
     SSHFactory = SSHUserAuthServer = SSHConnection = Key = SSHChannel = \
         SSHAgentServer = KnownHostsFile = SSHPublicKeyChecker = ConchUser = \
         object
@@ -709,6 +709,42 @@ class NewConnectionTests(TestCase, SSHCommandClientEndpointTestsMixin):
             self.factory, self.reactor.tcpClients[0][2])
 
 
+    def loseConnectionToServer(self, server, client, protocol, pump):
+        """
+        Lose the connection to a server and pump the L{IOPump} sufficiently for
+        the client to handle the lost connection. Asserts that the client
+        disconnects its transport.
+
+        @param server: The SSH server protocol over which C{protocol} is
+            running.
+        @type server: L{IProtocol} provider
+
+        @param client: The SSH client protocol over which C{protocol} is
+            running.
+        @type client: L{IProtocol} provider
+
+        @param protocol: The protocol created by calling connect on the ssh
+            endpoint under test.
+        @type protocol: L{IProtocol} provider
+
+        @param pump: The L{IOPump} connecting client to server.
+        @type pump: L{IOPump}
+        """
+        closed = self.record(server, protocol, 'closed', noArgs=True)
+        protocol.transport.loseConnection()
+        pump.pump()
+        self.assertEqual([None], closed)
+
+        # Let the last bit of network traffic flow.  This lets the server's
+        # close acknowledgement through, at which point the client can close
+        # the overall SSH connection.
+        pump.pump()
+
+        # Given that the client transport is disconnecting, report the
+        # disconnect from up to the ssh protocol.
+        client.transport.reportDisconnect()
+
+
     def assertClientTransportState(self, client, immediateClose):
         """
         Assert that the transport for the given protocol has been disconnected.
@@ -1104,7 +1140,8 @@ class NewConnectionTests(TestCase, SSHCommandClientEndpointTestsMixin):
         """
         If L{SSHCommandClientEndpoint} is initialized with an
         L{SSHAgentClient}, the agent is used to authenticate with the SSH
-        server.
+        server. Once the connection with the SSH server has concluded, the
+        connection to the agent is disconnected.
         """
         key = Key.fromString(privateRSA_openssh)
         agentServer = SSHAgentServer()
@@ -1137,6 +1174,12 @@ class NewConnectionTests(TestCase, SSHCommandClientEndpointTestsMixin):
         protocol = self.successResultOf(connected)
         self.assertIsNot(None, protocol.transport)
 
+        # Ensure the connection with the agent is cleaned up after the
+        # connection with the server is lost.
+        self.loseConnectionToServer(server, client, protocol, pump)
+        self.assertTrue(client.transport.disconnecting)
+        self.assertTrue(agentEndpoint.pump.clientIO.disconnecting)
+
 
     def test_loseConnection(self):
         """
@@ -1154,15 +1197,7 @@ class NewConnectionTests(TestCase, SSHCommandClientEndpointTestsMixin):
         server, client, pump = self.finishConnection()
 
         protocol = self.successResultOf(connected)
-        closed = self.record(server, protocol, 'closed', noArgs=True)
-        protocol.transport.loseConnection()
-        pump.pump()
-        self.assertEqual([None], closed)
-
-        # Let the last bit of network traffic flow.  This lets the server's
-        # close acknowledgement through, at which point the client can close
-        # the overall SSH connection.
-        pump.pump()
+        self.loseConnectionToServer(server, client, protocol, pump)
 
         # Nothing useful can be done with the connection at this point, so the
         # endpoint should close it.
