@@ -15,7 +15,6 @@ import sys
 import textwrap
 import tempfile
 import shutil
-import tarfile
 
 from datetime import date
 from io import BytesIO as StringIO
@@ -28,14 +27,13 @@ from twisted.python import release
 from twisted.python.filepath import FilePath
 from twisted.python.versions import Version
 
-from twisted.web.microdom import parseXMLString
 from twisted.python._release import (
     _changeVersionInFile, getNextVersion, findTwistedProjects, replaceInFile,
     replaceProjectVersion, Project, generateVersionFileData,
     changeAllProjectVersions, VERSION_OFFSET, filePathDelta, CommandFailed,
-    DistributionBuilder, APIBuilder, BuildAPIDocsScript, buildAllTarballs,
-    runCommand, UncleanWorkingDirectory, NotWorkingDirectory,
-    ChangeVersionsScript, BuildTarballsScript, NewsBuilder, SphinxBuilder,
+    APIBuilder, BuildAPIDocsScript,
+    runCommand, NotWorkingDirectory,
+    ChangeVersionsScript, NewsBuilder, SphinxBuilder,
     GitCommand, SVNCommand, getRepositoryCommand, IVCSCommand)
 
 if os.name != 'posix':
@@ -187,24 +185,6 @@ class StructureAssertingMixin(object):
                       % (root.path, children))
 
 
-    def assertExtractedStructure(self, outputFile, dirDict):
-        """
-        Assert that a tarfile content is equivalent to one described by a dict.
-
-        @param outputFile: The tar file built by L{DistributionBuilder}.
-        @type outputFile: L{FilePath}.
-        @param dirDict: The dict that should describe the contents of the
-            directory. It should be the same structure as the C{dirDict}
-            parameter to L{createStructure}.
-        @type dirDict: C{dict}
-        """
-        tarFile = tarfile.TarFile.open(outputFile.path, "r:bz2")
-        extracted = FilePath(self.mktemp())
-        for info in tarFile:
-            tarFile.extract(info, path=extracted.path)
-        self.assertStructure(extracted.children()[0], dirDict)
-
-
 
 class ChangeVersionTests(ExternalTempdirTestCase, StructureAssertingMixin):
     """
@@ -353,7 +333,7 @@ class ChangeVersionTests(ExternalTempdirTestCase, StructureAssertingMixin):
         """
         root = FilePath(self.mktemp())
         structure = {
-            "README": "Hi this is 1.0.0.",
+            "README.rst": "Hi this is 1.0.0.",
             "twisted": {
                 "topfiles": {
                     "README": "Hi this is 1.0.0"},
@@ -364,7 +344,7 @@ class ChangeVersionTests(ExternalTempdirTestCase, StructureAssertingMixin):
         releaseDate = date(2010, 1, 1)
         changeAllProjectVersions(root, False, False, releaseDate)
         outStructure = {
-            "README": "Hi this is 10.0.0.",
+            "README.rst": "Hi this is 10.0.0.",
             "twisted": {
                 "topfiles": {
                     "README": "Hi this is 10.0.0"},
@@ -389,7 +369,7 @@ class ChangeVersionTests(ExternalTempdirTestCase, StructureAssertingMixin):
                    "==================================\n"
                    "\n")
         structure = {
-            "README": "Hi this is 1.0.0pre1.",
+            "README.rst": "Hi this is 1.0.0pre1.",
             "NEWS": coreNews + webNews,
             "twisted": {
                 "topfiles": {
@@ -411,7 +391,7 @@ class ChangeVersionTests(ExternalTempdirTestCase, StructureAssertingMixin):
                    "==============================\n"
                    "\n")
         outStructure = {
-            "README": "Hi this is 1.0.0.",
+            "README.rst": "Hi this is 1.0.0.",
             "NEWS": coreNews + webNews,
             "twisted": {
                 "topfiles": {
@@ -1417,9 +1397,8 @@ class SphinxBuilderTests(TestCase):
 
     def verifyFileExists(self, fileDir, fileName):
         """
-        Helper which verifies that C{fileName} exists in C{fileDir}, has some
-        content, and that the content is parseable by L{parseXMLString} if the
-        file extension indicates that it should be html.
+        Helper which verifies that C{fileName} exists in C{fileDir} and it has
+        some content.
 
         @param fileDir: A path to a directory.
         @type fileDir: L{FilePath}
@@ -1436,8 +1415,7 @@ class SphinxBuilderTests(TestCase):
                 2. Is empty.
 
                 3. In the case where it's a path to a C{.html} file, the
-                   contents at least look enough like HTML to parse according
-                   to microdom's generous criteria.
+                   content looks like an HTML file.
 
         @return: C{None}
         """
@@ -1473,6 +1451,25 @@ class SphinxBuilderTests(TestCase):
         self.verifyBuilt()
 
 
+    def test_warningsAreErrors(self):
+        """
+        Creates and builds a fake Sphinx project as if via the command line,
+        failing if there are any warnings.
+        """
+        output = StringIO()
+        self.patch(sys, "stdout", output)
+        self.createFakeSphinxProject()
+        with self.sphinxDir.child("index.rst").open("a") as f:
+            f.write("\n.. _malformed-link-target\n")
+        exception = self.assertRaises(
+            SystemExit,
+            self.builder.main, [self.sphinxDir.parent().path]
+        )
+        self.assertEqual(exception.code, 1)
+        self.assertIn("malformed hyperlink target", output.getvalue())
+        self.verifyBuilt()
+
+
     def verifyBuilt(self):
         """
         Verify that a sphinx project has been built.
@@ -1501,283 +1498,6 @@ class SphinxBuilderTests(TestCase):
 
 
 
-class DistributionBuilderTestBase(StructureAssertingMixin,
-                                  ExternalTempdirTestCase):
-    """
-    Base for tests of L{DistributionBuilder}.
-    """
-
-    def setUp(self):
-        self.rootDir = FilePath(self.mktemp())
-        self.outputDir = FilePath(self.mktemp())
-        self.builder = DistributionBuilder(self.rootDir, self.outputDir)
-
-
-
-class DistributionBuilderTests(DistributionBuilderTestBase):
-
-    def test_twistedDistribution(self):
-        """
-        The Twisted tarball contains everything in the source checkout, with
-        built documentation.
-        """
-        manInput1 = "pretend there's some troff in here or something"
-        structure = {
-            "README": "Twisted",
-            "unrelated": "x",
-            "LICENSE": "copyright!",
-            "setup.py": "import toplevel",
-            "bin": {"web": {"websetroot": "SET ROOT"},
-                    "twistd": "TWISTD"},
-            "twisted": {
-                "web": {
-                    "__init__.py": "import WEB",
-                    "topfiles": {"setup.py": "import WEBINSTALL",
-                                 "README": "WEB!"}},
-                "words": {"__init__.py": "import WORDS"},
-                "plugins": {"twisted_web.py": "import WEBPLUG",
-                            "twisted_words.py": "import WORDPLUG"}},
-            "docs": {
-                "conf.py": testingSphinxConf,
-                "index.rst": "",
-                "core": {"man": {"twistd.1": manInput1}}
-            }
-        }
-
-        def hasManpagesAndSphinx(path):
-            self.assertTrue(path.isdir())
-            self.assertEqual(
-                path.child("core").child("man").child("twistd.1").getContent(),
-                manInput1
-            )
-            return True
-
-        outStructure = {
-            "README": "Twisted",
-            "unrelated": "x",
-            "LICENSE": "copyright!",
-            "setup.py": "import toplevel",
-            "bin": {"web": {"websetroot": "SET ROOT"},
-                    "twistd": "TWISTD"},
-            "twisted": {
-                "web": {"__init__.py": "import WEB",
-                        "topfiles": {"setup.py": "import WEBINSTALL",
-                                     "README": "WEB!"}},
-                "words": {"__init__.py": "import WORDS"},
-                "plugins": {"twisted_web.py": "import WEBPLUG",
-                            "twisted_words.py": "import WORDPLUG"}},
-            "doc": hasManpagesAndSphinx,
-        }
-
-        self.createStructure(self.rootDir, structure)
-
-        outputFile = self.builder.buildTwisted("10.0.0")
-
-        self.assertExtractedStructure(outputFile, outStructure)
-
-    test_twistedDistribution.skip = sphinxSkip
-
-
-    def test_excluded(self):
-        """
-        bin/admin and doc/historic are excluded from the Twisted tarball.
-        """
-        structure = {
-            "bin": {"admin": {"blah": "ADMIN"},
-                    "twistd": "TWISTD"},
-            "twisted": {
-                "web": {
-                    "__init__.py": "import WEB",
-                    "topfiles": {"setup.py": "import WEBINSTALL",
-                                 "README": "WEB!"}}},
-            "doc": {"historic": {"hello": "there"},
-                    "other": "contents"}}
-
-        outStructure = {
-            "bin": {"twistd": "TWISTD"},
-            "twisted": {
-                "web": {
-                    "__init__.py": "import WEB",
-                    "topfiles": {"setup.py": "import WEBINSTALL",
-                                 "README": "WEB!"}}},
-            "doc": {"other": "contents"}}
-
-        self.createStructure(self.rootDir, structure)
-        outputFile = self.builder.buildTwisted("10.0.0")
-        self.assertExtractedStructure(outputFile, outStructure)
-
-
-    def test_setup3(self):
-        """
-        setup3.py is included in the release tarball.
-        """
-        structure = {
-            "setup3.py": "install python 3 version",
-            "bin": {"twistd": "TWISTD"},
-            "twisted": {
-                "web": {
-                    "__init__.py": "import WEB",
-                    "topfiles": {"setup.py": "import WEBINSTALL",
-                                 "README": "WEB!"}}},
-            "doc": {"web": {"howto": {"index.html": "hello"}}},
-            }
-
-        self.createStructure(self.rootDir, structure)
-        outputFile = self.builder.buildTwisted("13.2.0")
-        self.assertExtractedStructure(outputFile, structure)
-
-
-
-class BuildAllTarballsTestBase(object):
-    """
-    Tests for L{DistributionBuilder.buildAllTarballs}.
-    """
-
-    def test_buildAllTarballs(self):
-        """
-        L{buildAllTarballs} builds tarballs for Twisted and all of its
-        subprojects based on a Git repository; the resulting tarballs contain
-        no Git metadata.  This involves building documentation, which it will
-        build with the correct API documentation reference base URL.
-        """
-        checkoutPath = self.mktemp()
-        checkout = FilePath(checkoutPath)
-        self.outputDir.remove()
-
-        self._init(checkout)
-
-        structure = {
-            "README": "Twisted",
-            "unrelated": "x",
-            "LICENSE": "copyright!",
-            "setup.py": "import toplevel",
-            "bin": {"words": {"im": "import im"},
-                    "twistd": "TWISTD"},
-            "twisted": {
-                "topfiles": {"setup.py": "import TOPINSTALL",
-                             "README": "CORE!"},
-                "_version.py": genVersion("twisted", 1, 2, 0),
-                "words": {"__init__.py": "import WORDS",
-                          "_version.py": genVersion("twisted.words", 1, 2, 0),
-                          "topfiles": {"setup.py": "import WORDSINSTALL",
-                                       "README": "WORDS!"}},
-                "plugins": {"twisted_web.py": "import WEBPLUG",
-                            "twisted_words.py": "import WORDPLUG",
-                            "twisted_yay.py": "import YAY"}},
-            "docs": {
-                "conf.py": testingSphinxConf,
-                "index.rst": "",
-            }
-        }
-
-        def smellsLikeSphinxOutput(actual):
-            self.assertTrue(actual.isdir())
-            self.assertIn("index.html", actual.listdir())
-            self.assertIn("objects.inv", actual.listdir())
-            return True
-
-        twistedStructure = {
-            "README": "Twisted",
-            "unrelated": "x",
-            "LICENSE": "copyright!",
-            "setup.py": "import toplevel",
-            "bin": {"twistd": "TWISTD",
-                    "words": {"im": "import im"}},
-            "twisted": {
-                "topfiles": {"setup.py": "import TOPINSTALL",
-                             "README": "CORE!"},
-                "_version.py": genVersion("twisted", 1, 2, 0),
-                "words": {"__init__.py": "import WORDS",
-                          "_version.py": genVersion("twisted.words", 1, 2, 0),
-                          "topfiles": {"setup.py": "import WORDSINSTALL",
-                                       "README": "WORDS!"}},
-                "plugins": {"twisted_web.py": "import WEBPLUG",
-                            "twisted_words.py": "import WORDPLUG",
-                            "twisted_yay.py": "import YAY"}},
-            "doc": smellsLikeSphinxOutput}
-
-        self.createStructure(checkout, structure)
-        childs = [x.path for x in checkout.children()]
-        self._addAndCommit(checkout, childs)
-
-        buildAllTarballs(checkout, self.outputDir)
-        self.assertEqual(
-            set(self.outputDir.children()),
-            set([self.outputDir.child("Twisted-1.2.0.tar.bz2")]))
-
-        self.assertExtractedStructure(
-            self.outputDir.child("Twisted-1.2.0.tar.bz2"),
-            twistedStructure)
-
-
-    def test_buildAllTarballsEnsuresCleanCheckout(self):
-        """
-        L{UncleanWorkingDirectory} is raised by L{buildAllTarballs} when the
-        Git repository provided has uncommitted changes.
-        """
-        checkoutPath = self.mktemp()
-        checkout = FilePath(checkoutPath)
-
-        self._init(checkout)
-
-        checkout.child("foo").setContent("whatever")
-        self.assertRaises(UncleanWorkingDirectory,
-                          buildAllTarballs, checkout, FilePath(self.mktemp()))
-
-
-    def test_buildAllTarballsEnsuresExistingCheckout(self):
-        """
-        L{NotWorkingDirectory} is raised by L{buildAllTarballs} when the
-        checkout passed does not exist or is not a Git repository.
-        """
-        checkout = FilePath(self.mktemp()).child("test")
-        self.assertRaises(NotWorkingDirectory,
-                          buildAllTarballs,
-                          checkout, FilePath(self.mktemp()))
-        checkout.createDirectory()
-        self.assertRaises(NotWorkingDirectory,
-                          buildAllTarballs,
-                          checkout, FilePath(self.mktemp()))
-
-
-
-class BuildAllTarballsGitTestCase(DistributionBuilderTestBase,
-                                  BuildAllTarballsTestBase):
-    """
-    Tests for L{DistributionBuilder.buildAllTarballs} using Git.
-    """
-    skip = gitSkip or sphinxSkip
-
-    def _init(self, directory):
-        _gitInit(directory)
-
-    def _addAndCommit(self, checkout, files):
-        runCommand(["git", "-C", checkout.path, "add", "-f"] + files)
-        runCommand(["git", "-C", checkout.path, "commit", "-m", "yay"])
-
-
-
-class BuildAllTarballsSVNTestCase(DistributionBuilderTestBase,
-                                  BuildAllTarballsTestBase):
-    """
-    Tests for L{DistributionBuilder.buildAllTarballs} using SVN.
-    """
-    skip = svnSkip or sphinxSkip
-
-    def _init(self, directory):
-        repositoryPath = self.mktemp()
-        repository = FilePath(repositoryPath)
-
-        runCommand(["svnadmin", "create", repository.path])
-        runCommand(["svn", "checkout", "file://" + repository.path,
-                    directory.path])
-
-    def _addAndCommit(self, checkout, files):
-        runCommand(["svn", "add"] + files)
-        runCommand(["svn", "commit", checkout.path, "-m", "yay"])
-
-
-
 class ScriptTests(StructureAssertingMixin, ExternalTempdirTestCase):
     """
     Tests for the release script functionality.
@@ -1868,52 +1588,6 @@ class ScriptTests(StructureAssertingMixin, ExternalTempdirTestCase):
                           ["my united.states.of prewhatever"])
 
 
-    def test_buildTarballsScript(self):
-        """
-        L{BuildTarballsScript.main} invokes L{buildAllTarballs} with
-        2 or 3 L{FilePath} instances representing the paths passed to it.
-        """
-        builds = []
-
-        def myBuilder(checkout, destination, template=None):
-            builds.append((checkout, destination, template))
-
-        tarballBuilder = BuildTarballsScript()
-        tarballBuilder.buildAllTarballs = myBuilder
-
-        tarballBuilder.main(["checkoutDir", "destinationDir"])
-        self.assertEqual(
-            builds,
-            [(FilePath("checkoutDir"), FilePath("destinationDir"), None)])
-
-        builds = []
-        tarballBuilder.main(["checkoutDir", "destinationDir", "templatePath"])
-        self.assertEqual(
-            builds,
-            [(FilePath("checkoutDir"), FilePath("destinationDir"),
-              FilePath("templatePath"))])
-
-
-    def test_defaultBuildTarballsScriptBuilder(self):
-        """
-        The default implementation of L{BuildTarballsScript.buildAllTarballs}
-        is L{buildAllTarballs}.
-        """
-        tarballBuilder = BuildTarballsScript()
-        self.assertEqual(tarballBuilder.buildAllTarballs, buildAllTarballs)
-
-
-    def test_badNumberOfArgumentsToBuildTarballs(self):
-        """
-        L{BuildTarballsScript.main} raises SystemExit when the wrong number of
-        arguments are passed.
-        """
-        tarballBuilder = BuildTarballsScript()
-        self.assertRaises(SystemExit, tarballBuilder.main, [])
-        self.assertRaises(SystemExit, tarballBuilder.main,
-                          ["a", "b", "c", "d"])
-
-
     def test_badNumberOfArgumentsToBuildNews(self):
         """
         L{NewsBuilder.main} raises L{SystemExit} when other than 1 argument is
@@ -2006,7 +1680,7 @@ class CommandsTestMixin(StructureAssertingMixin):
         repository as identical in a specified directory.
         """
         structure = {
-            "README": "Hi this is 1.0.0.",
+            "README.rst": "Hi this is 1.0.0.",
             "twisted": {
                 "topfiles": {
                     "README": "Hi this is 1.0.0"},
